local require = GLOBAL.require
require "mods"
local IsDST = GLOBAL.MOD_API_VERSION == 10

local function GetThePlayer()
  local player
  if IsDST then
    player = GLOBAL.ThePlayer
  else
    player = GLOBAL.GetPlayer()
  end
  return player
end

require("stategraph")

local state_broke_DS = GLOBAL.State{
  name = "weaponbroke",
  tags = {"busy"},
  onenter = function(inst, weapon)
    inst.AnimState:PlayAnimation("hit")
    inst.SoundEmitter:PlaySound("dontstarve/wilson/use_break")
    inst.AnimState:Hide("ARM_carry") 
    inst.AnimState:Show("ARM_normal") 
    local brokentool = GLOBAL.SpawnPrefab("brokentool")
    brokentool.Transform:SetPosition(inst.Transform:GetWorldPosition() )
    inst.sg.statemem.weapon = weapon
  end,

  onexit = function(inst)
    if inst.components.inventory then
      local sameWeapon = inst.components.inventory:FindItem(function(item)
        return item.prefab == inst.sg.statemem.weapon.prefab
      end)

      if sameWeapon then
        inst.components.inventory:Equip(sameWeapon)
      end

      if inst.components.inventory:GetEquippedItem(GLOBAL.EQUIPSLOTS.HANDS) then
        inst.AnimState:Show("ARM_carry") 
        inst.AnimState:Hide("ARM_normal")
      end
    end
  end,

  events=
  {
    GLOBAL.EventHandler("animover", function(inst)
      inst.sg:GoToState("idle")
    end ),
  },
}

local state_broke_DST = GLOBAL.State{
  name = "weaponbroke",
  tags = {"busy", "pausepredict"},
  onenter = function(inst, weapon)
    inst.AnimState:PlayAnimation("hit")
    inst.SoundEmitter:PlaySound("dontstarve/wilson/use_break")
    inst.AnimState:Hide("ARM_carry") 
    inst.AnimState:Show("ARM_normal") 
    GLOBAL.SpawnPrefab("brokentool").Transform:SetPosition(inst.Transform:GetWorldPosition())
    inst.sg.statemem.weapon = weapon

    if inst.components.playercontroller ~= nil then
      inst.components.playercontroller:RemotePausePrediction()
    end
  end,

  events =
  {
    GLOBAL.EventHandler("animover", function(inst)
      if inst.AnimState:AnimDone() then
        inst.sg:GoToState("idle")
      end
    end),
  },

  onexit = function(inst)
    if inst.components.inventory then
      local sameWeapon = inst.components.inventory:FindItem(function(item)
        return item.prefab == inst.sg.statemem.weapon.prefab
      end)
      if sameWeapon then
        inst.components.inventory:Equip(sameWeapon)
      end

      if inst.components.inventory:GetEquippedItem(GLOBAL.EQUIPSLOTS.HANDS) then
        inst.AnimState:Show("ARM_carry") 
        inst.AnimState:Hide("ARM_normal")
      end
    end
  end,
}

if IsDST then
  AddStategraphState("wilson", state_broke_DST)
else
  AddStategraphState("wilson", state_broke_DS)
end

local event_broke = GLOBAL.EventHandler("weaponbroke", function(inst, data) inst.sg:GoToState("weaponbroke", data.weapon) end)
AddStategraphEvent("wilson", event_broke)

local state_lost_DS = GLOBAL.State{
  name = "weaponlost",
  tags = {"busy"},
  onenter = function(inst, weapon)
    --inst.AnimState:PlayAnimation("hit")
    --inst.SoundEmitter:PlaySound("dontstarve/wilson/use_break")
    inst.AnimState:Hide("ARM_carry") 
    inst.AnimState:Show("ARM_normal") 
    --local brokentool = GLOBAL.SpawnPrefab("brokentool")
    --brokentool.Transform:SetPosition(inst.Transform:GetWorldPosition() )
    inst.sg.statemem.weapon = weapon
  end,

  onexit = function(inst)
    if inst.components.inventory then
      local sameWeapon = inst.sg.statemem.weapon
      if sameWeapon then
        inst.components.inventory:Equip(sameWeapon)
      end

      if inst.components.inventory:GetEquippedItem(GLOBAL.EQUIPSLOTS.HANDS) then
        inst.AnimState:Show("ARM_carry") 
        inst.AnimState:Hide("ARM_normal")
      end
    end
  end,

  events=
  {
    GLOBAL.EventHandler("animover", function(inst)
      inst.sg:GoToState("idle")
    end ),
  },
}

local state_lost_DST = GLOBAL.State{
  name = "weaponlost",
  tags = {"busy"},
  onenter = function(inst, weapon)
    --inst.AnimState:PlayAnimation("hit")
    --inst.SoundEmitter:PlaySound("dontstarve/wilson/use_break")
    inst.AnimState:Hide("ARM_carry") 
    inst.AnimState:Show("ARM_normal") 
    --local brokentool = GLOBAL.SpawnPrefab("brokentool")
    --brokentool.Transform:SetPosition(inst.Transform:GetWorldPosition() )
    inst.sg.statemem.weapon = weapon

    if inst.components.playercontroller ~= nil then
      inst.components.playercontroller:RemotePausePrediction()
    end
  end,

  events=
  {
    GLOBAL.EventHandler("animover", function(inst)
      inst.sg:GoToState("idle")
    end ),
  },

  onexit = function(inst)
    if inst.components.inventory then
      local sameWeapon = inst.sg.statemem.weapon
      if sameWeapon then
        inst.components.inventory:Equip(sameWeapon)
      end

      if inst.components.inventory:GetEquippedItem(GLOBAL.EQUIPSLOTS.HANDS) then
        inst.AnimState:Show("ARM_carry") 
        inst.AnimState:Hide("ARM_normal")
      end
    end
  end,
}

if IsDST then
  AddStategraphState("wilson", state_lost_DST)
else
  AddStategraphState("wilson", state_lost_DS)
end

local event_lost = GLOBAL.EventHandler("weaponlost", function(inst, data) inst.sg:GoToState("weaponlost", data.weapon) end)
AddStategraphEvent("wilson", event_lost)

local function AddAutoReEquipWeapon_weapon(self, inst)

  local function PercentChanged(inst, data)
    if inst.components.tool or inst.prefab == "torch" then return end
    --print("----------------------------------------------------------")
    --print("PercentChanged: inst.prefab", inst.prefab)
    if inst.components.weapon
       and inst.components.fueled
       and inst.components.inventoryitem
       and inst.components.inventoryitem.owner
       and inst.components.inventoryitem.owner.components.playercontroller
    then
      --print("inst.components.inventoryitem.owner=", inst.components.inventoryitem.owner)
      local player = inst.components.inventoryitem.owner
      local playercontroller = player.components.playercontroller
      --print("playercontroller.inst", playercontroller.inst)
      --print("playercontroller.inst.components.inventory", playercontroller.inst.components.inventory)
      if playercontroller.inst.components.inventory then
        samelostweapon = playercontroller.inst.components.inventory:FindItem(function(item)
          return item.prefab == inst.components.weapon.inst.prefab
        end)

        if playercontroller.inst
           and data.percent and math.floor(data.percent * 100) <= 0
        then
          inst.components.fueled:SetPercent(0) -- delete night stick
          playercontroller.inst:PushEvent("weaponlost", {weapon = samelostweapon})
          --print("PushEvent(weaponlost): inst.prefab", inst.prefab)
        end
      end
    elseif inst.components.weapon
           and data.percent and data.percent <= 0
           and inst.components.inventoryitem and inst.components.inventoryitem.owner
    then
      inst.components.inventoryitem.owner:PushEvent("weaponbroke", {weapon = inst})
      --print("PushEvent(weaponbroke): inst.prefab", inst.prefab)
    end
  end

  local samelostweapon = nil

  local function StackChanged(inst, data)
    if inst.components.tool or inst.prefab == "torch" then return end
    --print("----------------------------------------------------------")
    --print("StackChanged: inst.prefab", inst.prefab)
    --print("data.owner=",data.owner)
    if inst.components.weapon
       and data.owner
       and data.owner.components.playercontroller
    then
      local player = data.owner
      local playercontroller = player.components.playercontroller
      --print("playercontroller.inst", playercontroller.inst)
      --print("playercontroller.inst.components.inventory", playercontroller.inst.components.inventory)
      if playercontroller.inst and playercontroller.inst.components.inventory then
        samelostweapon = playercontroller.inst.components.inventory:FindItem(function(item)
          return item.prefab == inst.components.weapon.inst.prefab
        end)

        if data.stacksize and data.stacksize <= 0 then
          playercontroller.inst:PushEvent("weaponlost", {weapon = samelostweapon})
          --print("PushEvent(weaponlost): inst.prefab", inst.prefab)
        end
      end
    end
  end

  self.inst:ListenForEvent("percentusedchange", PercentChanged)
  self.inst:ListenForEvent("stackusedchange", StackChanged)

end

AddComponentPostInit("weapon", AddAutoReEquipWeapon_weapon)

local function AddAutoReEquipWeapon_projectile(self, inst)

  local projectile_miss_base = self.Miss
  self.Miss = function(self, target)
    --print("----------------------------------------------------------")
    --print("projectile_miss: self.owner=",self.owner)
    if self.inst.components.finiteuses then
      self.inst.components.finiteuses.owner = self.owner
    end
    if self.inst.components.stackable
       and self.owner
       and self.owner.components.playercontroller
   then
      local player = self.owner
      local playercontroller = player.components.playercontroller
      if playercontroller.inst and playercontroller.inst.components.inventory then
        local weapon = playercontroller.inst.components.inventory:GetEquippedItem(GLOBAL.EQUIPSLOTS.HANDS)
        local stacksize
        if weapon then
          if weapon.components.stackable then
            stacksize = weapon.components.stackable:StackSize()
          else
            stacksize = 1
          end
        else
          -- equip slot is empty
          stacksize = 0
        end
        self.inst:PushEvent("stackusedchange", {stacksize = stacksize, owner=self.owner})
        --print("PushEvent(stackusedchange): inst.prefab", inst.prefab)
      end
    end
    projectile_miss_base(self, target)
  end

  local projectile_hit_base = self.Hit
  self.Hit = function(self, target)
    --print("----------------------------------------------------------")
    --print("projectile_hit: self.owner=",self.owner)
    if self.inst.components.finiteuses then
      self.inst.components.finiteuses.owner = self.owner
    end
    if self.inst.components.stackable
       and self.owner
       and self.owner.components.playercontroller
    then
      local player = self.owner
      local playercontroller = player.components.playercontroller
      if playercontroller.inst and playercontroller.inst.components.inventory then
        local weapon = playercontroller.inst.components.inventory:GetEquippedItem(GLOBAL.EQUIPSLOTS.HANDS)
        local stacksize
        if weapon then
          if weapon.components.stackable then
            stacksize = weapon.components.stackable:StackSize()
          else
            stacksize = 1
          end
        else
          -- equip slot is empty
          stacksize = 0
        end
        self.inst:PushEvent("stackusedchange", {stacksize = stacksize,owner=self.owner})
        --print("PushEvent(stackusedchange): inst.prefab", inst.prefab)
      end
    end
    projectile_hit_base(self, target)
  end

end

AddComponentPostInit("projectile", AddAutoReEquipWeapon_projectile)

local function AddAutoReEquipWeapon_finiteuses(self, inst)

  local setonfinished_base = self.SetOnFinished
  self.SetOnFinished = function(self, fn)
    local onfinish_base = fn
    self.onfinished = function(inst)
      if inst.components.weapon
         and inst.components.projectile
      then
        -- boomerang
        inst:PushEvent("stackusedchange", {stacksize=0, owner=self.owner})
        --print("----------------------------------------------------------")
        --print("SetOnFinished: PushEvent(stackusedchange): inst.prefab", inst.prefab)
      end
      onfinish_base(inst)
    end
  end

end

AddComponentPostInit("finiteuses", AddAutoReEquipWeapon_finiteuses)

local function AddAutoReEquipWeapon_spellcaster(self, inst)

  local function PercentChanged(inst, data)
    if inst.components.tool or inst.prefab == "torch" then return end
    --print("----------------------------------------------------------")
    --print("PercentChanged: inst.prefab", inst.prefab)
    --print("data.owner=",data.owner)
    if inst.components.spellcaster
       and inst.components.fueled
       and data.owner
       and data.owner.components.playercontroller
    then
      local player = data.owner
      local playercontroller = player.components.playercontroller
      --print("playercontroller.inst", playercontroller.inst)
      --print("playercontroller.inst.components.inventory", playercontroller.inst.components.inventory)
      if playercontroller.inst and playercontroller.inst.components.inventory then
        samelostspellcaster = playercontroller.inst.components.inventory:FindItem(function(item)
          return item.prefab == inst.components.spellcaster.inst.prefab
        end)

        if playercontroller.inst
           and data.percent and math.floor(data.percent * 100) <= 0
        then
          inst.components.fueled:SetPercent(0)
          playercontroller.inst:PushEvent("weaponlost", {weapon = samelostspellcaster})
          --print("PushEvent(weaponlost): inst.prefab", inst.prefab)
        end
      end
    elseif inst.components.spellcaster
           and data.percent and data.percent <= 0
           and inst.components.inventoryitem and inst.components.inventoryitem.owner
    then
      inst.components.inventoryitem.owner:PushEvent("weaponbroke", {weapon = inst})
      --print("PushEvent(weaponbroke): inst.prefab", inst.prefab)
    end
  end

  self.inst:ListenForEvent("percentusedchange", PercentChanged)

end

AddComponentPostInit("spellcaster", AddAutoReEquipWeapon_spellcaster)

local function AddAutoReEquipWeapon_blinkstaff(self, inst)

  local function PercentChanged(inst, data)
    if inst.components.tool or inst.prefab == "torch" then return end
    --print("----------------------------------------------------------")
    --print("PercentChanged: inst.prefab", inst.prefab)
    if inst.components.blinkstaff
       and inst.components.fueled
       and data.owner
       and data.owner.components.playercontroller
    then
      local player = data.owner
      local playercontroller = player.components.playercontroller
      --print("playercontroller.inst", playercontroller.inst)
      --print("playercontroller.inst.components.inventory", playercontroller.inst.components.inventory)
      if playercontroller.inst and playercontroller.inst.components.inventory then
        samelostblinkstaff = playercontroller.inst.components.inventory:FindItem(function(item)
          return item.prefab == inst.components.blinkstaff.inst.prefab
        end)

        if playercontroller.inst
           and data.percent and math.floor(data.percent * 100) <= 0
        then
          inst.components.fueled:SetPercent(0)
          playercontroller.inst:PushEvent("weaponlost", {weapon = samelostblinkstaff})
          --print("PushEvent(weaponlost): inst.prefab", inst.prefab)
        end
      end
    elseif inst.components.blinkstaff
           and data.percent and data.percent <= 0
           and inst.components.inventoryitem and inst.components.inventoryitem.owner
    then
      inst.components.inventoryitem.owner:PushEvent("weaponbroke", {weapon = inst})
      --print("PushEvent(weaponbroke): inst.prefab", inst.prefab)
    end
  end

  self.inst:ListenForEvent("percentusedchange", PercentChanged)

end

AddComponentPostInit("blinkstaff", AddAutoReEquipWeapon_blinkstaff)
